##Empirical Application of Sakaguchi (2023) Using the Data of McCall (1996).

#Packages#
using DataFrames, Random, Distributions, Optim, Plots, ProgressBars, RoundingIntegers, CSV, Dates, LaTeXStrings
date = today()

###Installing data###
data = CSV.read("UnempDur.csv",DataFrame)

###Preliminary Values###
N = size(data)[1]
Rep = 500

###Define variables#
dur = data[!,"spell"]
event = ones(N) .- data[!,"censor4"]
ui= data[!,"ui"]
wage = data[!,"logwage"]

d = event
y0 = dur
y1 = d.*dur + (ones(N).-d).*50
x1 = wage
x2 = ui
y_base = median(y0)
p_d = 1-mean(d)

###PRE estimated and inference###

function PRE_func(b)
    A = 0
    for i in 1:N
        for j in 1:N
            A = A + (y1[i]>y0[j] && b[1]*(x1[i]-x1[j])+b[2]*(x2[i]-x2[j])>0)*1
        end
    end
    return -A/(N*(N-1))
end

##PRE estimation##
@time res = optimize(PRE_func,[-0.1,0.4])
estimate = -Optim.minimizer(res)[2]/Optim.minimizer(res)[1]

##Bootstrap Inference##

function Bootstrap(R)
    B = zeros(R)
    for r in 1:R
        v = rand(1:N,N)
        d = event[v]
        y0 = dur[v]
        y1 = d.*y0 + (ones(N).-d).*50
        x1 = wage[v]
        x2 = ui[v]
        function MRC_func(b)
            A = 0
            for i in 1:N
                for j in 1:N
                    A = A + (y1[i]>y0[j] && b[1]*(x1[i]-x1[j])+b[2]*(x2[i]-x2[j])>0)*1
                end
            end
            return -A/(N*(N-1))
        end
        res = optimize(MRC_func,[-0.1,0.0])
        B[r] = -Optim.minimizer(res)[2]/Optim.minimizer(res)[1]
    end
    return [quantile(B,0.025),quantile(B,0.975)]
end

#Results#
@time result1 = [estimate,Bootstrap(Rep)]


####Proposed inference procedure####

###Inference Function for beta###
function Q_func(b,R,e)
    Ng = 0
    for r=1:R
        Ng = Ng + (2*r)^2*4
    end
    nom1 = Normal(mean(x1),std(x1))
    nom = Normal(0, 1)
    A = 0
    sigma1N = 0
    m1N = 0
    v = zeros(N)
    for i=1:N
        f = 0
        for j=1:N
            A = ((y1[i]>=y0[j] && b[1]*(x1[i]-x1[j])+b[2]*(x2[i]-x2[j])>0)*1
                + (y1[j]>y0[i] && b[1]*(x1[j]-x1[i])+b[2]*(x2[j]-x2[i])>0)*1)
            m1N = m1N + A
            f = f + A
        end
        v[i] = f/N
    end
    m1N = m1N/(N*(N-1))
    sigma1N = var(v)
    m2N = zeros(R,2*R,2*R,2,2)
    sigma2N = zeros(R,2*R,2*R,2,2)
    HR = zeros(R)
    for r=1:R
        Q = 0
        for k=1:(2*r), l=1:(2*r), m=0:1, n=0:1
            A = 0
            H = 0
            v = zeros(N)
            for i in 1:N
                f = 0
                if quantile(nom1,(k-1)/(2*r))<x1[i]<=quantile(nom1,k/(2*r)) && x2[i]==m
                    for j in 1:N
                        if quantile(nom1,(l-1)/(2*r))<x1[j]<=quantile(nom1,l/(2*r)) && x2[j]==n
                            A = (-1/2 + (y1[i]>=y0[j] && b[1]*(x1[i]-x1[j])+b[2]*(x2[i]-x2[j])>0)*1
                                + (y1[j]>y0[i] && b[1]*(x1[j]-x1[i])+b[2]*(x2[j]-x2[i])>0)*1)
                            H = H + A
                            f = f + A 
                        end 
                    end 
                end
                v[i] = f/N 
            end
            H = H/(N*(N-1))
            m2N[r,k,l,m+1,n+1] = H
            sigma2 = var(v)
            sigma2N[r,k,l,m+1,n+1] = sigma2
            Q = Q + ((H^2)/(sigma2 + e*sigma1N))*(H<0) 
        end    
        HR[r] = Q/(((2*r)^2)*4*(r^2+100))
    end
    cmi = zeros(Rep)
    Random.seed!(012)
    for rep=1:Rep
        v = rand(nom,Ng)
        HRQ = zeros(R)
        c = 0
        for r=1:R
            Q = 0
            for k=1:(2*r),l=1:(2*r),m=0:1,n=0:1
                c = c + 1
                H = (sqrt(sigma2N[r,k,l,m+1,n+1])*v[c]
                    + sqrt(sigma1N)*sqrt(0.8*log(N)/log(log(N)))*(sqrt(N)*m2N[r,k,l,m+1,n+1]/(sqrt(sigma2N[r,k,l,m+1,n+1] + e*sigma1N)*sqrt(((1-p_d^(1/3))^(2/5))*0.6*log(N)))>1)*1)
                Q = Q + ((H^2)/(sigma2N[r,k,l,m+1,n+1] + e*sigma1N))*(H<0)*1
            end
            HRQ[r] = Q/(((2*r)^2)*4*(r^2+100))
        end
        cmi[rep] = sum(HRQ)
    end
    C = zeros(2)
    C[1] = N*sum(HR)
    C[2] = quantile(cmi,0.950001)
    return [C[1],C[2],(C[1]<C[2])*1]
end

##Function to compute an identification set##
function idset_func(b2_grid,R,e)
    b2_grid_n = length(b2_grid)
    result = 0; lower_bound = 0; upper_bound = 0
    for j=1:b2_grid_n        
        result = Q_func([-0.1,b2_grid[j]],R,e)[3]
        if result == 1
            lower_bound = b2_grid[j]/0.1
            break        
        end        
    end
    for j=1:b2_grid_n        
        result = Q_func([-0.1,b2_grid[b2_grid_n-j+1]],R,e)[3]
        if result == 1
            upper_bound = b2_grid[b2_grid_n-j+1]/0.1
            break        
        end        
    end
    return [[lower_bound,upper_bound]]
end


@time result2 = idset_func([0.11:0.001:5;],5,0.1)

@time result3 = idset_func([0.08:0.001:5;],5,0.3)

print("PRE estimate:",result1[1],"\n","PRE CI:",result1[2],"\n","Proposed method CI with epsilon=0.1:",result2[1],"\n","Proposed method CI with epsilon=0.3:",result3[1])









